package cn.com.qmth.examcloud.core.oe.student.face.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.concurrency.simple.Worker;
import cn.com.qmth.examcloud.commons.helpers.concurrency.simple.WorkerController;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamCaptureQueueEntity;
import cn.com.qmth.examcloud.core.oe.student.face.service.ExamCaptureService;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 百度活体检测工作线程
 *
 * @author lideyin 20190620
 */
public class BaiduFaceLivenessWorker implements Worker {
    private final Log captureLog = LogFactory.getLog("PROCESS_EXAM_CAPTURE_TASK_LOGGER");

    @Override
    public void process(WorkerController controller, Object element) {
        ExamCaptureProcessStatisticController.increaseFaceLivenessDetectCount();
        if (captureLog.isDebugEnabled()){
            captureLog.debug("[BAIDU_FACELIVENESS_WORKER.] 图片处理数量+1，count= " + ExamCaptureProcessStatisticController.getFaceLivenessDetectCount());
        }
        ExamCaptureService examCaptureService = SpringContextHolder.getBean(ExamCaptureService.class);
        ExamCaptureQueueEntity examCaptureQueue = (ExamCaptureQueueEntity) element;
        try {
            examCaptureService.disposeBaiDuFaceLiveness(examCaptureQueue);
        } catch (StatusException e) {

            //异常处理
            examCaptureQueue.setErrorMsg(e.getDesc());
            examCaptureService.disposeBaiDuFaceLivenessFaild(examCaptureQueue);
            if ((e.getCode().equals(Constants.BAIDU_FACELIVENESS_QPS_LIMIT_EXCEEDED_CODE))) {
                // 如果超过并发次数，则添加异常次数
                controller.addConcurrencyWarn();
            } else {
                ExamCaptureProcessStatisticController.increaseFaceLivenessDetectCount();
                captureLog.error("[BAIDU_FACELIVENESS_WORKER.] 自定义异常 " + e.getDesc(), e);
            }
        } catch (Exception e) {
            ExamCaptureProcessStatisticController.increaseFaceLivenessDetectCount();
            //异常处理
            examCaptureQueue.setErrorMsg(e.getMessage());
            examCaptureService.disposeBaiDuFaceLivenessFaild(examCaptureQueue);
            captureLog.error("[BAIDU_FACELIVENESS_WORKER.] 系统异常 " + e.getMessage(), e);
        }
    }
}
