package cn.com.qmth.examcloud.core.oe.student.face.service.impl;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @Description 考试照片统计控制器
 * @Author lideyin
 * @Date 2019/9/9 17:05
 * @Version 1.0
 */
public class ExamCaptureProcessStatisticController {
    //人脸比对处理数量
    private static AtomicInteger faceCompareCount = new AtomicInteger(0);
    //人脸比对处理失败数量
    private static AtomicInteger faceCompareFailedCount = new AtomicInteger(0);

    private static ReentrantLock facePPLock = new ReentrantLock();
    private static ReentrantLock faceLivenessLock = new ReentrantLock();

    /**
     * 人脸比对总数量自增
     */
    public static void increaseFaceCompareCount() {
        faceCompareCount.incrementAndGet();
    }

    /**
     * 人脸比对错误数量自增
     */
    public static void increaseFaceCompareFailedCount() {
        faceCompareFailedCount.incrementAndGet();
    }

    /**
     * 同步重置人脸比对所有数量
     */
    public static void resetAllFaceCompareCount() {
        try {
            facePPLock.lock();
            faceCompareCount.set(0);
            faceCompareFailedCount.set(0);
        } finally {
            facePPLock.unlock();
        }
    }

    /**
     * 人脸比对总数量
     *
     * @return int
     */
    public static int getFaceCompareCount() {
        return faceCompareCount.get();
    }

    /**
     * 人脸比对失败数量
     *
     * @return int
     */
    public static int getFaceCompareFailedCount() {
        return faceCompareFailedCount.get();
    }

    /**
     * 人脸比对失败率
     *
     * @return double
     */
    public static double getFaceCompareFailureRate() {
        if (faceCompareCount.get() == 0) {
            return 0;
        }
        return (double) faceCompareFailedCount.get() / (double) faceCompareCount.get();
    }

    //活体检测处理数量
    private static AtomicInteger faceLivenessDetectCount = new AtomicInteger(0);
    //活体检测处理失败数量
    private static AtomicInteger faceLivenessDetectFailedCount = new AtomicInteger(0);

    /**
     * 活体检测总数量自增
     */
    public static void increaseFaceLivenessDetectCount() {
        faceLivenessDetectCount.incrementAndGet();
    }

    /**
     * 活体检测错误数量自增
     */
    public static void increaseFaceLivenessDetectFailedCount() {
        faceLivenessDetectFailedCount.incrementAndGet();
    }

    /**
     * 重置活体检测所有数量
     */
    public static void resetAllFaceLivenessDetectCount() {
        try {
            faceLivenessLock.lock();
            faceLivenessDetectCount.set(0);
            faceLivenessDetectFailedCount.set(0);
        } finally {
            faceLivenessLock.unlock();
        }
    }

    /**
     * 活体检测总数量
     *
     * @return
     */
    public static int getFaceLivenessDetectCount() {
        return faceLivenessDetectCount.get();
    }

    /**
     * 活体检测失败数量
     *
     * @return
     */
    public static int getFaceLivenessDetectFailedCount() {
        return faceLivenessDetectFailedCount.get();
    }

    /**
     * 活体检测失败率
     *
     * @return
     */
    public static double getFaceLivenessDetectFailureRate() {
        if (faceLivenessDetectCount.get() == 0) {
            return 0;
        }
        return (double) faceLivenessDetectFailedCount.get() / (double) faceLivenessDetectCount.get();
    }
}
