package cn.com.qmth.examcloud.core.oe.student.face.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.base.utils.HttpPoolUtil;
import cn.com.qmth.examcloud.core.oe.student.face.service.FaceCompareService;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.net.URI;

/**
 * @author chenken
 * @date 2018年10月30日 下午5:17:28
 * @company QMTH
 * @description 百度活体检测执行service
 */
@Service("faceCompareService")
public class FaceCompareServiceImpl implements FaceCompareService {
    private static final Logger log = LoggerFactory.getLogger(FaceCompareServiceImpl.class);

    @Value("${$facepp.apiKey}")
    private String faceppApiKey;
    @Value("${$facepp.apiSecret}")
    private String faceppApiSecret;

    @Override
    public JSONObject getFaceppCompareResultByUrl(String baseFaceToken, String image_url2) {

        CloseableHttpClient httpClient = HttpPoolUtil.getHttpClient();
        CloseableHttpResponse httpResponse = null;

        HttpPost httpPost = new HttpPost(getFaceCompareUri());
        log.debug("[getFaceppCompareResultByUrl] .getHost=" + getFaceCompareUri().getHost());
        MultipartEntityBuilder multipartEntityBuilder = MultipartEntityBuilder.create();
        multipartEntityBuilder.addTextBody("api_key", faceppApiKey);
        multipartEntityBuilder.addTextBody("api_secret", faceppApiSecret);
        multipartEntityBuilder.addTextBody("face_token1", baseFaceToken);
        multipartEntityBuilder.addTextBody("image_url2", image_url2);
        HttpEntity httpEntity = multipartEntityBuilder.build();

        httpPost.setEntity(httpEntity);
        String result = "";
        try {
            httpResponse = httpClient.execute(httpPost);
            HttpEntity responseEntity = httpResponse.getEntity();
            BufferedReader reader = new BufferedReader(new InputStreamReader(responseEntity.getContent()));
            StringBuffer buffer = new StringBuffer();
            String str = "";
            while (StringUtils.isNoneBlank((str = reader.readLine()))) {
                buffer.append(str);
            }
            result = buffer.toString();
        } catch (Exception e) {
            log.error("facepp call failed", e);
            JSONObject resultJson = new JSONObject();
            String errorMsg = e.toString();
            if (errorMsg != null && errorMsg.length() > 500) {
                errorMsg = errorMsg.substring(0, 500);
            }
            resultJson.put(Constants.ERROR_MSG, "facepp call failed:" + errorMsg);
            result = resultJson.toString();
        } finally {
            try {
                if (httpResponse != null) {
                    httpResponse.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        log.debug("[getFaceppCompareResultByUrl] .result=" + result);
        return JSONObject.parseObject(result);
    }

    /**
     * 通过face++得到人脸比较结果
     *
     * @param baseFaceToken
     * @param file
     * @return
     */
    @Override
    public JSONObject getFaceppCompareResultByFile(String baseFaceToken, File file) {
//		CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpClient httpClient = HttpPoolUtil.getHttpClient();
        CloseableHttpResponse httpResponse = null;
        HttpPost httpPost = new HttpPost(getFaceCompareUri());
        MultipartEntityBuilder multipartEntityBuilder = MultipartEntityBuilder.create();
        multipartEntityBuilder.addTextBody("api_key", faceppApiKey);
        multipartEntityBuilder.addTextBody("api_secret", faceppApiSecret);
        multipartEntityBuilder.addTextBody("face_token1", baseFaceToken);
        multipartEntityBuilder.addBinaryBody("image_file2", file, ContentType.IMAGE_JPEG, file.getName());
        HttpEntity httpEntity = multipartEntityBuilder.build();

        httpPost.setEntity(httpEntity);
        String result = "";
        try {
            httpResponse = httpClient.execute(httpPost);
            HttpEntity responseEntity = httpResponse.getEntity();
            BufferedReader reader = new BufferedReader(new InputStreamReader(responseEntity.getContent()));
            StringBuffer buffer = new StringBuffer();
            String str = "";
            while (StringUtils.isNoneBlank((str = reader.readLine()))) {
                buffer.append(str);
            }
            result = buffer.toString();
        } catch (Exception e) {
            log.error("facepp call failed", e);
            JSONObject resultJson = new JSONObject();
            String errorMsg = e.toString();
            if (errorMsg != null && errorMsg.length() > 500) {
                errorMsg = errorMsg.substring(0, 500);
            }
            resultJson.put(Constants.ERROR_MSG, "facepp call failed:" + errorMsg);
            result = resultJson.toString();
        } finally {
            try {
                if (httpResponse != null) {
                    httpResponse.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return JSONObject.parseObject(result);
    }

    /**
     * 获取人脸比对uri
     *
     * @return
     */
    private URI getFaceCompareUri() {
        SysPropertyCacheBean sysProperty = CacheHelper.getSysProperty("facePlusPlus.faceCompare.url");
        if (sysProperty.getHasValue()) {
            Object value = sysProperty.getValue();
            if (null != value) {
                String url = value.toString();
                log.debug("[getFaceCompareUri] .sysProperty.getValue()=" + sysProperty.getValue());
                System.setProperty("facepp.compare.url",url);
                return URI.create(url);
            }
        }
        throw new StatusException("300004", "未找到face++人脸比对的配置路径");
    }
}
