package cn.com.qmth.examcloud.core.oe.student.face.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.concurrency.simple.Worker;
import cn.com.qmth.examcloud.commons.helpers.concurrency.simple.WorkerController;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamCaptureQueueEntity;
import cn.com.qmth.examcloud.core.oe.student.face.service.ExamCaptureService;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * face++人脸比对工作线程
 *
 * @author lideyin 20190620
 */
public class FacePPCompareWorker implements Worker {
    private final Log captureLog = LogFactory.getLog("PROCESS_EXAM_CAPTURE_TASK_LOGGER");

    @Override
    public void process(WorkerController controller, Object element) {
        //图片处理数量+1
        ExamCaptureProcessStatisticController.increaseFaceCompareCount();
        if (captureLog.isDebugEnabled()){
            captureLog.debug("[FACEPP_COMPARE_WORKER.] 图片处理数量+1，count= " + ExamCaptureProcessStatisticController.getFaceCompareCount());
        }
        ExamCaptureService examCaptureService = SpringContextHolder
                .getBean(ExamCaptureService.class);
        ExamCaptureQueueEntity examCaptureQueueEntity = (ExamCaptureQueueEntity) element;
        try {
            examCaptureService.disposeFaceCompare(examCaptureQueueEntity);
        } catch (StatusException e) {
            //异常处理
            examCaptureQueueEntity.setErrorMsg(e.getDesc());
            examCaptureService.disposeFaceCompareFaild(examCaptureQueueEntity);
            if ((e.getCode().equals(Constants.FACE_COMPARE_CONCURRENCY_LIMIT_EXCEEDED))) {
                // 如果超过并发次数，则添加异常次数
                controller.addConcurrencyWarn();
            } else {
                ExamCaptureProcessStatisticController.increaseFaceCompareFailedCount();
                captureLog.error("[FACEPP_COMPARE_WORKER.] 自定义异常 " + e.getDesc(), e);
            }
        } catch (Exception e) {
            ExamCaptureProcessStatisticController.increaseFaceCompareFailedCount();
            //异常处理
            examCaptureQueueEntity.setErrorMsg(e.getMessage());
            examCaptureService.disposeFaceCompareFaild(examCaptureQueueEntity);
            captureLog.error("[FACEPP_COMPARE_WORKER.] 系统异常 " + e.getMessage(), e);
        }
    }
}
